mod\_authn\_core

Core Authentication

Base

mod\_authn\_core.c

authn\_core\_module

Available in Apache 2.3 and later

This module provides core authentication capabilities to allow or deny
access to portions of the web site.

mod\_authn\_core

provides directives that are common to all authentication providers.

Creating Authentication Provider Aliases {#authnalias}
========================================

Extended authentication providers can be created within the
configuration file and assigned an alias name. The alias providers can
then be referenced through the directives

AuthBasicProvider

or

AuthDigestProvider

in the same way as a base authentication provider. Besides the ability
to create and alias an extended provider, it also allows the same
extended authentication provider to be reference by multiple locations.

Examples {#example}
--------

This example checks for passwords in two different text files.

\# Check here first \<AuthnProviderAlias file file1\> AuthUserFile
\"/www/conf/passwords1\" \</AuthnProviderAlias\> \# Then check here
\<AuthnProviderAlias file file2\> AuthUserFile \"/www/conf/passwords2\"
\</AuthnProviderAlias\> \<Directory \"/var/web/pages/secure\"\>
AuthBasicProvider file1 file2 AuthType Basic AuthName \"Protected Area\"
Require valid-user \</Directory\>

The example below creates two different ldap authentication provider
aliases based on the ldap provider. This allows a single authenticated
location to be serviced by multiple ldap hosts:

\<AuthnProviderAlias ldap ldap-alias1\> AuthLDAPBindDN
\"cn=youruser,o=ctx\" AuthLDAPBindPassword yourpassword AuthLDAPURL
\"ldap://ldap.host/o=ctx\" \</AuthnProviderAlias\> \<AuthnProviderAlias
ldap ldap-other-alias\> AuthLDAPBindDN \"cn=yourotheruser,o=dev\"
AuthLDAPBindPassword yourotherpassword AuthLDAPURL
\"ldap://other.ldap.host/o=dev?cn\" \</AuthnProviderAlias\> Alias
\"/secure\" \"/webpages/secure\" \<Directory \"/webpages/secure\"\>
AuthBasicProvider ldap-other-alias ldap-alias1 AuthType Basic AuthName
\"LDAP Protected Place\" Require valid-user \# Note that Require ldap-\*
would not work here, since the \# AuthnProviderAlias does not provide
the config to authorization providers \# that are implemented in the
same module as the authentication provider. \</Directory\>

AuthName

Authorization realm for use in HTTP authentication

AuthName

auth-domain

directory

.htaccess

AuthConfig

This directive sets the name of the authorization realm for a directory.
This realm is given to the client so that the user knows which username
and password to send.

AuthName

takes a single argument; if the realm name contains spaces, it must be
enclosed in quotation marks. It must be accompanied by

AuthType

and

Require

directives, and directives such as

AuthUserFile

and

AuthGroupFile

to work.

For example:

AuthName \"Top Secret\"

The string provided for the

AuthName

is what will appear in the password dialog provided by most browsers.

From 2.5.0,

expression syntax

can be used inside the directive to produce the name dynamically.

For example:

AuthName \"%{HTTP\_HOST}\"

Authentication, Authorization, and Access Control

mod\_authz\_core

AuthType

Type of user authentication

AuthType None\|Basic\|Digest\|Form

directory

.htaccess

AuthConfig

This directive selects the type of user authentication for a directory.
The authentication types available are

None

,

Basic

(implemented by

mod\_auth\_basic

),

Digest

(implemented by

mod\_auth\_digest

), and

Form

(implemented by

mod\_auth\_form

).

To implement authentication, you must also use the

AuthName

and

Require

directives. In addition, the server must have an authentication-provider
module such as

mod\_authn\_file

and an authorization module such as

mod\_authz\_user

.

The authentication type

None

disables authentication. When authentication is enabled, it is normally
inherited by each subsequent

configuration section

, unless a different authentication type is specified. If no
authentication is desired for a subsection of an authenticated section,
the authentication type

None

may be used; in the following example, clients may access the

/www/docs/public

directory without authenticating:

\<Directory \"/www/docs\"\> AuthType Basic AuthName Documents
AuthBasicProvider file AuthUserFile
\"/usr/local/apache/passwd/passwords\" Require valid-user \</Directory\>
\<Directory \"/www/docs/public\"\> AuthType None Require all granted
\</Directory\>

From 2.5.0,

expression syntax

can be used inside the directive to specify the type dynamically.

> **Note**
>
> When disabling authentication, note that clients which have already
> authenticated against another portion of the server\'s document tree
> will typically continue to send authentication HTTP headers or cookies
> with each request, regardless of whether the server actually requires
> authentication for every resource.

Authentication, Authorization, and Access Control

AuthnProviderAlias

Enclose a group of directives that represent an extension of a base
authentication provider and referenced by the specified alias

\<AuthnProviderAlias

baseProvider Alias

\> \... \</AuthnProviderAlias\>

server config

\<AuthnProviderAlias\>

and

\</AuthnProviderAlias\>

are used to enclose a group of authentication directives that can be
referenced by the alias name using one of the directives

AuthBasicProvider

or

AuthDigestProvider

.

> **Note**
>
> This directive has no affect on authorization, even for modules that
> provide both authentication and authorization.
