mod\_isapi

ISAPI Extensions within Apache for Windows

Base

mod\_isapi.c

isapi\_module

Win32 only

This module implements the Internet Server extension API. It allows
Internet Server extensions (

e.g.

ISAPI .dll modules) to be served by Apache for Windows, subject to the
noted restrictions.

ISAPI extension modules (.dll files) are written by third parties. The
Apache Group does not author these modules, so we provide no support for
them. Please contact the ISAPI\'s author directly if you are
experiencing problems running their ISAPI extension.

Please

do not

post such problems to Apache\'s lists or bug reporting pages.

Usage
=====

In the server configuration file, use the

AddHandler

directive to associate ISAPI files with the

isapi-handler

handler, and map it to them with their file extensions. To enable any
.dll file to be processed as an ISAPI extension, edit the httpd.conf
file and add the following line:

AddHandler isapi-handler .dll

> **Note**
>
> In older versions of the Apache server,
>
> isapi-isa
>
> was the proper handler name, rather than
>
> isapi-handler
>
> . As of 2.3 development versions of the Apache server,
>
> isapi-isa
>
> is no longer valid. You will need to change your configuration to use
>
> isapi-handler
>
> instead.

There is no capability within the Apache server to leave a requested
module loaded. However, you may preload and keep a specific module
loaded by using the following syntax in your httpd.conf:

ISAPICacheFile c:/WebWork/Scripts/ISAPI/mytest.dll

Whether or not you have preloaded an ISAPI extension, all ISAPI
extensions are governed by the same permissions and restrictions as CGI
scripts. That is,

Options

ExecCGI

must be set for the directory that contains the ISAPI .dll file.

Review the

Additional Notes

and the

Programmer\'s Journal

for additional details and clarification of the specific ISAPI support
offered by

mod\_isapi

.

Additional Notes {#notes}
================

Apache\'s ISAPI implementation conforms to all of the ISAPI 2.0
specification, except for some \"Microsoft-specific\" extensions dealing
with asynchronous I/O. Apache\'s I/O model does not allow asynchronous
reading and writing in a manner that the ISAPI could access. If an ISA
tries to access unsupported features, including async I/O, a message is
placed in the error log to help with debugging. Since these messages can
become a flood, the directive

ISAPILogNotSupported Off

exists to quiet this noise.

Some servers, like Microsoft IIS, load the ISAPI extension into the
server and keep it loaded until memory usage is too high, or unless
configuration options are specified. Apache currently loads and unloads
the ISAPI extension each time it is requested, unless the

ISAPICacheFile

directive is specified. This is inefficient, but Apache\'s memory model
makes this the most effective method. Many ISAPI modules are subtly
incompatible with the Apache server, and unloading these modules helps
to ensure the stability of the server.

Also, remember that while Apache supports ISAPI Extensions, it

does not support ISAPI Filters

. Support for filters may be added at a later date, but no support is
planned at this time.

Programmer\'s Journal {#journal}
=====================

If you are programming Apache 2.0

mod\_isapi

modules, you must limit your calls to

ServerSupportFunction

to the following directives:

HSE\_REQ\_SEND\_URL\_REDIRECT\_RESP

Redirect the user to another location.

This must be a fully qualified URL (

e.g.

http://server/location

).

HSE\_REQ\_SEND\_URL

Redirect the user to another location.

This cannot be a fully qualified URL, you are not allowed to pass the
protocol or a server name (

e.g.

simply

/location

).

This redirection is handled by the server, not the browser.

> **Note**
>
> In their recent documentation, Microsoft appears to have abandoned the
> distinction between the two
>
> HSE\_REQ\_SEND\_URL
>
> functions. Apache continues to treat them as two distinct functions
> with different requirements and behaviors.

HSE\_REQ\_SEND\_RESPONSE\_HEADER

Apache accepts a response body following the header if it follows the
blank line (two consecutive newlines) in the headers string argument.
This body cannot contain NULLs, since the headers argument is NULL
terminated.

HSE\_REQ\_DONE\_WITH\_SESSION

Apache considers this a no-op, since the session will be finished when
the ISAPI returns from processing.

HSE\_REQ\_MAP\_URL\_TO\_PATH

Apache will translate a virtual name to a physical name.

HSE\_APPEND\_LOG\_PARAMETER

This logged message may be captured in any of the following logs:

in the

\\\"%{isapi-parameter}n\\\"

component in a

CustomLog

directive

in the

\%q

log component with the

ISAPIAppendLogToQuery

On

directive

in the error log with the

ISAPIAppendLogToErrors

On

directive

The first option, the

\%{isapi-parameter}n

component, is always available and preferred.

HSE\_REQ\_IS\_KEEP\_CONN

Will return the negotiated Keep-Alive status.

HSE\_REQ\_SEND\_RESPONSE\_HEADER\_EX

Will behave as documented, although the

fKeepConn

flag is ignored.

HSE\_REQ\_IS\_CONNECTED

Will report false if the request has been aborted.

Apache returns

FALSE

to any unsupported call to

ServerSupportFunction

, and sets the

GetLastError

value to

ERROR\_INVALID\_PARAMETER

.

ReadClient

retrieves the request body exceeding the initial buffer (defined by

ISAPIReadAheadBuffer

). Based on the

ISAPIReadAheadBuffer

setting (number of bytes to buffer prior to calling the ISAPI handler)
shorter requests are sent complete to the extension when it is invoked.
If the request is longer, the ISAPI extension must use

ReadClient

to retrieve the remaining request body.

WriteClient

is supported, but only with the

HSE\_IO\_SYNC

flag or no option flag (value of

0

). Any other

WriteClient

request will be rejected with a return value of

FALSE

, and a

GetLastError

value of

ERROR\_INVALID\_PARAMETER

.

GetServerVariable

is supported, although extended server variables do not exist (as
defined by other servers.) All the usual Apache CGI environment
variables are available from

GetServerVariable

, as well as the

ALL\_HTTP

and

ALL\_RAW

values.

Since httpd 2.0,

mod\_isapi

supports additional features introduced in later versions of the ISAPI
specification, as well as limited emulation of async I/O and the

TransmitFile

semantics. Apache httpd also supports preloading ISAPI .dlls for
performance.

ISAPICacheFile

ISAPI .dll files to be loaded at startup

ISAPICacheFile

file-path

\[

file-path

\] \...

server config

virtual host

Specifies a space-separated list of file names to be loaded when the
Apache server is launched, and remain loaded until the server is shut
down. This directive may be repeated for every ISAPI .dll file desired.
The full path name of each file should be specified. If the path name is
not absolute, it will be treated relative to

ServerRoot

.

ISAPIReadAheadBuffer

Size of the Read Ahead Buffer sent to ISAPI extensions

ISAPIReadAheadBuffer

size

ISAPIReadAheadBuffer 49152

server config

virtual host

directory

.htaccess

FileInfo

Defines the maximum size of the Read Ahead Buffer sent to ISAPI
extensions when they are initially invoked. All remaining data must be
retrieved using the

ReadClient

callback; some ISAPI extensions may not support the

ReadClient

function. Refer questions to the ISAPI extension\'s author.

ISAPILogNotSupported

Log unsupported feature requests from ISAPI extensions

ISAPILogNotSupported on\|off

ISAPILogNotSupported off

server config

virtual host

directory

.htaccess

FileInfo

Logs all requests for unsupported features from ISAPI extensions in the
server error log. This may help administrators to track down problems.
Once set to on and all desired ISAPI modules are functioning, it should
be set back to off.

ISAPIAppendLogToErrors

Record

HSE\_APPEND\_LOG\_PARAMETER

requests from ISAPI extensions to the error log

ISAPIAppendLogToErrors on\|off

ISAPIAppendLogToErrors off

server config

virtual host

directory

.htaccess

FileInfo

Record

HSE\_APPEND\_LOG\_PARAMETER

requests from ISAPI extensions to the server error log.

ISAPIAppendLogToQuery

Record

HSE\_APPEND\_LOG\_PARAMETER

requests from ISAPI extensions to the query field

ISAPIAppendLogToQuery on\|off

ISAPIAppendLogToQuery on

server config

virtual host

directory

.htaccess

FileInfo

Record

HSE\_APPEND\_LOG\_PARAMETER

requests from ISAPI extensions to the query field (appended to the

CustomLog

\%q

component).

ISAPIFakeAsync

Fake asynchronous support for ISAPI callbacks

ISAPIFakeAsync on\|off

ISAPIFakeAsync off

server config

virtual host

directory

.htaccess

FileInfo

While set to on, asynchronous support for ISAPI callbacks is simulated.
