mod\_session\_cookie

Cookie based session support

Extension

mod\_session\_cookie.c

session\_cookie\_module

Available in Apache 2.3 and later

> **Note**
>
> The session modules make use of HTTP cookies, and as such can fall
> victim to Cross Site Scripting attacks, or expose potentially private
> information to clients. Please ensure that the relevant risks have
> been taken into account before enabling the session functionality on
> your server.

This submodule of

mod\_session

provides support for the storage of user sessions on the remote browser
within HTTP cookies.

Using cookies to store a session removes the need for the server or a
group of servers to store the session locally, or collaborate to share a
session, and can be useful for high traffic environments where a server
based session might be too resource intensive.

If session privacy is required, the

mod\_session\_crypto

module can be used to encrypt the contents of the session before writing
the session to the client.

For more details on the session interface, see the documentation for the

mod\_session

module.

mod\_session

mod\_session\_crypto

mod\_session\_dbd

Basic Examples {#basicexamples}
==============

To create a simple session and store it in a cookie called

session

, configure the session as follows:

Session On SessionCookieName session path=/

For more examples on how the session can be configured to be read from
and written to by a CGI application, see the

mod\_session

examples section.

For documentation on how the session can be used to store username and
password details, see the

mod\_auth\_form

module.

SessionCookieName

Name and attributes for the RFC2109 cookie storing the session

SessionCookieName

name

attributes

none

server config

virtual host

directory

.htaccess

AuthConfig

The

SessionCookieName

directive specifies the name and optional attributes of an RFC2109
compliant cookie inside which the session will be stored. RFC2109
cookies are set using the

Set-Cookie

HTTP header.

An optional list of cookie attributes can be specified, as per the
example below. These attributes are inserted into the cookie as is, and
are not interpreted by Apache. Ensure that your attributes are defined
correctly as per the cookie specification.

Session On SessionCookieName session
path=/private;domain=example.com;httponly;secure;version=1;

SessionCookieName2

Name and attributes for the RFC2965 cookie storing the session

SessionCookieName2

name

attributes

none

server config

virtual host

directory

.htaccess

AuthConfig

The

SessionCookieName2

directive specifies the name and optional attributes of an RFC2965
compliant cookie inside which the session will be stored. RFC2965
cookies are set using the

Set-Cookie2

HTTP header.

An optional list of cookie attributes can be specified, as per the
example below. These attributes are inserted into the cookie as is, and
are not interpreted by Apache. Ensure that your attributes are defined
correctly as per the cookie specification.

Session On SessionCookieName2 session
path=/private;domain=example.com;httponly;secure;version=1;

SessionCookieRemove

Control for whether session cookies should be removed from incoming HTTP
headers

SessionCookieRemove On\|Off

SessionCookieRemove Off

server config

virtual host

directory

.htaccess

AuthConfig

The

SessionCookieRemove

flag controls whether the cookies containing the session will be removed
from the headers during request processing.

In a reverse proxy situation where the Apache server acts as a server
frontend for a backend origin server, revealing the contents of the
session cookie to the backend could be a potential privacy violation.
When set to on, the session cookie will be removed from the incoming
HTTP headers.

SessionCookieMaxAge

Control whether session cookies have Max-Age transmitted to the client

SessionCookieMaxAge On\|Off

SessionCookieMaxAge On

server config

virtual host

directory

.htaccess

AuthConfig

The

SessionCookieMaxAge

flag controls whether the session expiration will be specified in the
Max-Age attribute on the cookie sent to the client. When set to \'Off\',
the attribute will not be added and clients will only return the cookie
until \"the current session is over\". This often means until the
browser is closed.

The expiration of the session is still validated on the server by the

SessionMaxAge

directive.
